@@ -1,6 +1,7 @@
 import asyncio
 import collections
 import datetime
+import random
 import traceback
 from queue import Queue
 from io import StringIO
@@ -13,10 +14,12 @@ from pychess.Players.Player import PlayerIsDead, PassInterrupt, TurnInterrupt, I
 from pychess.System import conf
 from pychess.System.protoopen import protoopen, protosave
 from pychess.System.Log import log
+from pychess.Utils.book import getOpenings
 from pychess.Utils.Move import Move
 from pychess.Utils.eco import get_eco
 from pychess.Utils.Offer import Offer
 from pychess.Utils.TimeModel import TimeModel
+from pychess.Savers import html, txt
 from pychess.Variants.normal import NormalBoard
 
 from .logic import getStatus, isClaimableDraw, playerHasMatingMaterial
@@ -198,7 +201,7 @@ class GameModel(GObject.GObject):
             return self.starting_color
 
     def zero_reached(self, timemodel, color):
-        if conf.get('autoCallFlag') and self.players[1 - color].__type__ == ARTIFICIAL:
+        if conf.get('autoCallFlag'):
             if self.status == RUNNING and timemodel.getPlayerTime(color) <= 0:
                 log.info(
                     'Automatically sending flag call on behalf of player %s.' %
@@ -633,17 +636,41 @@ class GameModel(GObject.GObject):
         if error:
             raise error
 
-    def save(self, uri, saver, append, position=None):
-        if isinstance(uri, str):
+    def save(self, uri, saver, append, position=None, flip=False):
+        if saver in (html, txt):
+            fileobj = open(uri, "a" if append else "w", encoding="utf-8", newline="")
+            self.uri = uri
+        elif isinstance(uri, str):
             fileobj = protosave(uri, append)
             self.uri = uri
         else:
             fileobj = uri
             self.uri = None
-        saver.save(fileobj, self, position)
+        saver.save(fileobj, self, position, flip)
         self.needsSave = False
         self.emit("game_saved", uri)
 
+    def get_book_move(self):
+        openings = getOpenings(self.boards[-1].board)
+        openings.sort(key=lambda t: t[1], reverse=True)
+        if not openings:
+            return None
+
+        total_weights = 0
+        for move, weight, learn in openings:
+            total_weights += weight
+
+        if total_weights < 1:
+            return None
+
+        choice = random.randint(0, total_weights - 1)
+
+        current_sum = 0
+        for move, weight, learn in openings:
+            current_sum += weight
+            if current_sum > choice:
+                return Move(move)
+
     # Run stuff
 
     def start(self):
@@ -682,8 +709,14 @@ class GameModel(GObject.GObject):
             if not self.lesson_game:
                 self.checkStatus()
 
+            if self.isEngine2EngineGame() and self.timed:
+                self.timemodel.start()
+                self.timemodel.started = True
+
             self.curColor = self.boards[-1].color
 
+            book_depth_max = conf.get("book_depth_max")
+
             while self.status in (PAUSED, RUNNING, DRAW, WHITEWON, BLACKWON):
                 curPlayer = self.players[self.curColor]
                 if self.timed:
@@ -695,12 +728,21 @@ class GameModel(GObject.GObject):
                 try:
                     log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: calling %s.makeMove()" % (
                         id(self), str(self.players), self.ply, str(curPlayer)))
-                    if self.ply > self.lowply:
-                        move = yield from curPlayer.makeMove(self.boards[-1], self.moves[-1], self.boards[-2])
-                    else:
-                        move = yield from curPlayer.makeMove(self.boards[-1], None, None)
-                    log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: got move=%s from %s" % (
-                        id(self), str(self.players), self.ply, move, str(curPlayer)))
+
+                    move = None
+                    if curPlayer.__type__ == ARTIFICIAL and book_depth_max > 0 and self.ply <= book_depth_max:
+                        move = self.get_book_move()
+                        log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: got move=%s from book" % (
+                            id(self), str(self.players), self.ply, move))
+                        if move is not None:
+                            curPlayer.set_board(self.boards[-1].move(move))
+                    if move is None:
+                        if self.ply > self.lowply:
+                            move = yield from curPlayer.makeMove(self.boards[-1], self.moves[-1], self.boards[-2])
+                        else:
+                            move = yield from curPlayer.makeMove(self.boards[-1], None, None)
+                        log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: got move=%s from %s" % (
+                            id(self), str(self.players), self.ply, move, str(curPlayer)))
                 except PlayerIsDead as e:
                     if self.status in (WAITING_TO_START, PAUSED, RUNNING):
                         stringio = StringIO()
@@ -746,6 +788,7 @@ class GameModel(GObject.GObject):
                 newBoard = self.boards[-1].move(move)
                 newBoard.board.prev = self.boards[-1].board
 
+                # newBoard.printPieces()
                 # Variation on next move can exist from the hint panel...
                 if self.boards[-1].board.next is not None:
                     newBoard.board.children = self.boards[
