@@ -109,7 +109,10 @@ def start_lesson_from(filename, index=None):
     progress = lessons_solving_progress.get(filename, [0] * chessfile.count)
 
     if index is None:
-        index = progress.index(0)
+        try:
+            index = progress.index(0)
+        except ValueError:
+            index = 0
 
     rec = records[index]
 
